using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using ConsoleUtil;

namespace Hl7Service
{
    public class ServiceModule : ServiceBase
    {
        // This has to be static so that we can access it in the ControlCHandler.  See comments
        // with the ControlCHandler for more details.
        private static ListenerModule serverInstance;
        private ConsoleCtrl consoleControl;

        public ServiceModule()
        {
            
        }

        protected override void OnStart(string[] args)
        {
            // We call the real method - this allows us to call the DoStartImp in Debug console mode
            OnStartImp(true);
        }

        // This method has to be static since it is called by the operating system
        // when the user does control+C in the debug mode for this application.  The serverInstance
        // is a singleton (i.e. only one instance in the process) which means we can invoke StopHl7Server
        // to stop the service when we are running in debug mode as a console process.
        public static void ControlCHandler(ConsoleCtrl.ConsoleEvent consoleEvent)
        {
            if (ConsoleCtrl.ConsoleEvent.CTRL_C == consoleEvent)
            {
                System.Console.WriteLine("Got Control+C");
                serverInstance.StopHl7Server();
            }
        }

        private void MakeControlCHandler()
        {
            consoleControl = new ConsoleCtrl();
            consoleControl.ControlEvent += new ConsoleCtrl.ControlEventHandler(ControlCHandler);
        }

        public void OnStartImp(bool runAsService)
        {
            // As described in the manual http://www.interfaceware.com/manual/llp_service.html,
            // we need to spawn a thread to handle the networking events since we need to run a
            // message loop which is blocking.
            serverInstance = new ListenerModule(runAsService);
            Thread serverThread = new Thread(serverInstance.StartHl7Server);

            if (!runAsService)
            {
                MakeControlCHandler();
            }

            serverThread.Start();
        }

        protected override void OnStop()
        {
            // Stop the HL7 Server.  This calls StopMessageLoop() which sends a POST_QUIT
            // message to the objects in the message loop.
            serverInstance.StopHl7Server();
        }
    }
}
